"""
Fields used by easymode's i18n.meta package which modifies a class to enable i18n.
"""
from django.conf import settings
from django.utils import translation
from django.utils.encoding import force_unicode
from django.utils.translation.trans_real import translation as translation_catalogs

from easymode.i18n.meta.value import GettextVO
from easymode.i18n.meta.utils import get_localized_property, valid_for_gettext, \
    get_fallback_languages, get_localized_field_name
from easymode.utils.languagecode import get_real_fieldname
from easymode.utils.standin import standin_for

class DefaultFieldDescriptor(property):
    """
    Descriptor that implements access to the default language.
    It inherits property because django wants that, it doesn't process
    it as a field if it is not isinstance(self, property)
    """

    def __init__(self, name, **kwargs):
        """The name param is the name of the field the descriptor is emulating."""
        self.name = name
        self.__dict__.update(kwargs)

    def __get__(self, obj, typ=None):
        """
        Read the localised version of the field this descriptor emulates.
        First try to see if the localised field is really set.
        If not, then use ugettext_lazy to find a tranlation in the current language
        for this field.
        """
        # self must be returned in a getattr context.
        if obj is None:
            return self

        current_language = translation.get_language()
        real_field_name = get_real_fieldname(self.name, current_language)

        vo = GettextVO()
        
        # first check if the database contains the localized data
        vo.stored_value = getattr(obj, real_field_name)

        # the database does not have our localized data.
        # check if we have a translation, first get the msgid, as a unicode string.
        vo.msgid = get_localized_property(obj, self.name, getattr(settings, 'MSGID_LANGUAGE', settings.LANGUAGE_CODE))

        # check the translation in the current language
        # but avoid empty string and None 
        if valid_for_gettext(vo.msgid):
            vo.msg = self.to_python(translation.ugettext(force_unicode(vo.msgid)))
        elif valid_for_gettext(vo.stored_value):
            # we can not use the msgid for gettext but we did find a valid
            # translation in the database. Fine we stop here and return that
            # value. No need for a standin, because we don't have a catalog value.
            return vo.stored_value
        else:
            # we can not use the msgid for gettext lookups, so there is no
            # point in trying. Check for fallback languages in database.
            vo.fallback = get_localized_property(obj, self.name)
            
            if not valid_for_gettext(vo.fallback):
                # Also if we are sure we don't have any old
                # translations in the catalog or something for the fallback
                # languages in the database, we do not need to return a standin either
                return vo.msgid
        
        # we got here so we've got a valid messageid. Now collect data from the catalog(s)
        
        # if there isn't anything new in the catalog belonging to the current language:
        if vo.msg == vo.msgid:
            # maybe we have a translation in any of the fallback languages.
            if hasattr(settings, 'FALLBACK_LANGUAGES'):
                # first check if the database has the localized data in
                # any of the fallback languages.
                vo.fallback = get_localized_property(obj, self.name)
                
                # if the fallback is the same as the msgid, go and look in the catalog
                if vo.fallback == vo.msgid:
                    # there might be a translation in any
                    # of the fallback languages.
                    for fallback in get_fallback_languages():
                        catalog = translation_catalogs(fallback)
                        msg = catalog.ugettext(force_unicode(vo.msgid))
                        if self.to_python(msg) != vo.msgid:
                            vo.fallback = self.to_python(msg)
                            break
                elif vo.fallback:
                    # if a valid fallback is found, then, since the msg is equal
                    # to the msgid, the fallback is the winner.
                    vo.msg = vo.fallback

        # if we came here we collected data from the catalog and we should return
        # a standin. A standin is the return value, with some extra properties.
        # see GettextVO for the extra properties added.
        if valid_for_gettext(vo.stored_value):
            vo.standin_value_is_from_database = True
            # database always wins
            return standin_for(vo.stored_value, **vo.__dict__)
        elif valid_for_gettext(vo.msg):
            # runner up is the translation in the native language
            return standin_for(vo.msg, **vo.__dict__)
        elif valid_for_gettext(vo.fallback):
            # and last is the translation in a fallback language
            return standin_for(vo.fallback, **vo.__dict__)

        assert(valid_for_gettext(vo.msgid))

        # there is a very very small probability that the translation of
        # a valid msgid evaluates to empty string or None (after to_python).
        # If that happened, we got here. I choose to return None, because i like
        # to be like that
        return None

    def __set__(self, obj, value):
        """Write the localised version of the field this descriptor emulates."""
        local_property = get_localized_field_name(obj, self.name)
        setattr(obj, local_property, value)

    def get_internal_type(self):
        """Used to describe the type while serializing."""
        return "CharField"

    def value_to_string(self, obj):
        """This descriptor acts as a Field, as far as the serializer is concerned."""
        try:
            return force_unicode(self.__get__(obj))
        except TypeError:
            return str(self.__get__(obj))



"""
Contains widgets that can be used for admin models
with related items.
"""
from django import forms
from django.conf import settings
from django.core import urlresolvers
from django.forms import widgets
from django.template.loader import render_to_string
from django.utils.encoding import force_unicode
from django.utils.html import mark_safe
from django.utils.translation import ugettext

from easymode.utils.languagecode import strip_language_code

__all__ = ('RenderLink', 'LinkWidget')

class RenderLink(forms.Widget):
    """
    Renders a link to an admin page based on the primary key
    value of the model.
    """
    input_type = None
    
    def _has_changed(self, initial, data):
        return False
    def id_for_label(self, id_):
        return "hmm geen idee"
        
    def render(self, name, value, attrs=None):
        
        modelname = self.attrs['modelname']
        app_label = self.attrs['app_label']
        label = self.attrs['label']
        url_pattern = '%s:%s_%s_change' % ('admin', app_label, modelname)
        
        url = strip_language_code(urlresolvers.reverse(url_pattern, args=[value]))
            
        if value is None: value = ''
        final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        if value != '':
            # Only add the 'value' attribute if a value is non-empty.
            final_attrs['value'] = force_unicode(value)
        return render_to_string('tree/admin/widgets/foreignkeylink.html', locals())


class LinkWidget(widgets.TextInput):
    """
    A widget that renders only a link the change view or edit view of an object.
    """
    def _has_changed(self, initial, data):
        return False

    def render(self, name, value, attrs=None):
        # dress up the widget as a field, so it can be rendered by 'admin/includes/fieldset.html'
        fieldset = [[self]]

        if value is None:
            value = ''
        if value != '':
            value = force_unicode(self._format_value(value))

        # store values, they will be needed in self.field later on
        self.value = value
        self.name = name
        
        return render_to_string('admin/includes/fieldset.html', {'fieldset':[[self,]]})

    def field(self):
        # the whole widget is based entirely on the fact that *render* gets called
        # BEFORE *field*. And that is true because field is called by the rendering
        # done in *render*.
        context = {
            'name':self.name,
            'value':self.value,
            'description': ugettext('Change'),
            'ADMIN_MEDIA_PREFIX':settings.ADMIN_MEDIA_PREFIX,
        }
        
        context.update(self.attrs)
        if self.attrs.get('popup', False):
            return render_to_string('tree/admin/widgets/add_widget.html', context)
        
        return render_to_string('tree/admin/widgets/link_widget.html', context)

    def label_tag(self):
        return mark_safe(u'<label class="required">%s</label>' % self.attrs['label'] )




from hashlib import md5

from django.test import TestCase

from easymode.tree import xml as tree
from easymode.tests.models import TestModel, TestGenericFkModel, TagModel
from easymode.tests.testcases import initdb
from easymode.tree.xml.serializers import RecursiveXmlSerializer


__all__ = ('RecursiveSerializerTest',)

CONFIRMED_XML_DIGEST = 'ed0fcf77a8dab1f708874cb77aaf1666'

@initdb
class RecursiveSerializerTest(TestCase):
    """
    Test the functionality related to
    xmlserializable models.
    """
    fixtures = ['tests-testmodel.xml']
    
    def test_create_test_models(self):
        """create test models"""
        assert(TestModel.objects.all().count() == 1)
        
    def test_tree_serialize(self):
        """test serialization of models with foreign keys"""
        ser = RecursiveXmlSerializer()
        res = TestModel.objects.all()
        xml = ser.serialize(res)
        
        assert(md5(xml).hexdigest() == CONFIRMED_XML_DIGEST)
        
    def test_model_to_xml(self):
        """test the __xml__ method added to the model"""
        xml = TestModel.objects.get(pk=1).__xml__()
        assert( md5(xml).hexdigest() == CONFIRMED_XML_DIGEST)
     
    def test_model_to_xml_with_xml_function(self):
        """test the xml function in package tree"""
        xml = tree.xml(TestModel.objects.get(pk=1))
        assert( md5(xml).hexdigest() == CONFIRMED_XML_DIGEST)
        
    def test_query_set_to_xml(self):
        """Test the __xml__ method on querysets"""
        xml = TestModel.objects.all().__xml__()
        assert( md5(xml).hexdigest() == CONFIRMED_XML_DIGEST)
    
    def test_generic_relations_also_work(self):
        """Generic relations should also be converted to xml properly"""
        r = TestGenericFkModel(name='ik ben cool')
        r.save()
        r.relateds.create(name='ik ben graaf')
        r.save()
        xml = TestGenericFkModel.objects.all().__xml__()
        assert(xml.index( 'ik ben cool') != -1)
        assert(xml.index( 'ik ben graaf') != -1)
        r.relateds.create(name='ik lust spruitjes')
        r.save()
        xml = TestGenericFkModel.objects.all().__xml__()
        assert(xml.index( 'ik lust spruitjes') != -1)
    
    def test_natural_keys(self):
        "natural keys will show up in the xml"
        
        xml_with_natural_keys = tree.xml(TestModel.objects.all())
        assert(xml_with_natural_keys.find('<natural>In lectus est, viverra a, ultricies ut, pulvinar vitae, tellus.</natural>') != -1)
    
    def test___serialize__(self):
        "A model with a __serialize__ method should serialize it self"
        tags = TagModel.objects.all().order_by('id')
        ser = RecursiveXmlSerializer()
        self.assertEqual(ser.serialize(tags), '<django-objects version="1.0"><taggy>good</taggy><taggy>bad</taggy><taggy>ugly</taggy></django-objects>')

    def test_many_to_many_field(self):
        "ManyToManyField should be followed in one direction, like foreign keys"
        
        data = tree.xml(TestModel.objects.all())
        assert(data.index('<taggy>good</taggy>')!= -1)
        assert(data.index('<taggy>ugly</taggy>')!= -1)
        
        bad_tag = TagModel.objects.get(value_en='bad')
        first_item = TestModel.objects.get(pk=1)
        first_item.tags.add(bad_tag)
        first_item.save()
        data = tree.xml(TestModel.objects.all())
        assert(data.index('<taggy>bad</taggy>')!= -1)



"""
Contains an instance of Serializer and extends it to allow for recusive
serialization of django models with foreign keys.
"""
import sys
from StringIO import StringIO

from django.conf import settings
from django.core.serializers import xml_serializer
from django.utils.encoding import smart_unicode, force_unicode

from easymode.tree.introspection import get_default_field_descriptors, \
    get_foreign_key_desciptors, get_generic_relation_descriptors
from easymode.utils import recursion_depth
from easymode.utils.xmlutils import XmlPrinter


MANY_TO_MANY_RECURSION_LIMIT_ERROR = """
Deep recursion in easymode.tree.xml.serializers.RecursiveXmlSerializer:
Serializer found ManyToManyField named: %s from %s to %s.

You probably built a cycle in your model tree using many to many relations.
Please inspect the stack trace and try to fix the problem.

If you are sure these is no cycle and you just have lots and lots of models
with a gigantic deep tree, you can also try to set settings.RECURSION_LIMIT to
a value greater then 100.
"""

# startDocument must only be called once
# so override it
def startDocumentOnlyOnce(self):
    if hasattr(self, 'allready_called'):
        return
    self.allready_called = True
    XmlPrinter.startDocument(self)

XmlPrinter.startDocument = startDocumentOnlyOnce

class RecursiveXmlSerializer(xml_serializer.Serializer):
    """
    Serializes a queryset including related fields
    """
    def serialize(self, queryset, **options):
        """
        Serialize a queryset.
        THE OUTPUT OF THIS SERIALIZER IS NOT MEANT TO BE SERIALIZED BACK
        INTO THE DB.        
        """
        self.options = options

        self.stream = options.get("stream", StringIO())
        self.selected_fields = options.get("fields")
        self.use_natural_keys = options.get("use_natural_keys", True)
        
        self.xml = options.get("xml", None)
        self.root = (self.xml == None)
        
        self.start_serialization()
        for obj in queryset:
            # hook for having custom serialization
            if hasattr(obj, '__serialize__'):
                obj.__serialize__(self.xml)
            else:
                self.serialize_object(obj)
        self.end_serialization()
        return self.getvalue()
    
    def serialize_object(self, obj):
        """
        Write one item to the object stream
        """
        self.start_object(obj)
        for field in obj._meta.local_fields:
            if field.serialize and getattr(field, 'include_in_xml', True):
                if field.rel is None:
                    if self.selected_fields is None or field.attname in self.selected_fields:
                        self.handle_field(obj, field)
                else:
                    if self.selected_fields is None or field.attname[:-3] in self.selected_fields:
                        self.handle_fk_field(obj, field)

        # recursively serialize all foreign key relations
        for (foreign_key_descriptor_name, foreign_key_descriptor ) in get_foreign_key_desciptors(obj):
            # don't follow foreign keys that have a 'nofollow' attribute
            if foreign_key_descriptor.related.field.serialize \
                and not hasattr(foreign_key_descriptor.related.field, 'nofollow'):
                bound_foreign_key_descriptor = foreign_key_descriptor.__get__(obj)
                s = RecursiveXmlSerializer()                
                s.serialize( bound_foreign_key_descriptor.all(), xml=self.xml, stream=self.stream)

        #recursively serialize all one to one relations
        # TODO: make this work for non abstract inheritance but without infinite recursion
        # for (one_to_one_descriptor_name, one_to_one_descriptor) in get_one_to_one_descriptors(obj):
        #     related_objects = []
        #     try:
        #         related_object = one_to_one_descriptor.__get__(obj)
        #         related_objects.append(related_object)
        #     except Exception as e:
        #         pass
        # 
        #     s = RecursiveXmlSerializer()                
        #     s.serialize( related_objects, xml=self.xml, stream=self.stream)

        # add generic relations
        for (generic_relation_descriptor_name, generic_relation_descriptor) in get_generic_relation_descriptors(obj):
            # generic relations always have serialize set to False so we always include them.
            bound_generic_relation_descriptor = generic_relation_descriptor.__get__(obj)
            s = RecursiveXmlSerializer()                
            s.serialize( bound_generic_relation_descriptor.all(), xml=self.xml, stream=self.stream)
        
        #serialize the default field descriptors:
        for (default_field_descriptor_name, default_field_descriptor) in get_default_field_descriptors(obj):
            if default_field_descriptor.serialize:
                self.handle_field(obj, default_field_descriptor)
        
        for field in obj._meta.many_to_many:
            if field.serialize:
                if self.selected_fields is None or field.attname in self.selected_fields:
                    self.handle_m2m_field(obj, field)
        self.end_object(obj)
        
    def start_serialization(self):
        """
        Start serialization -- open the XML document and the root element.
        """
        if (self.root):
            self.xml = XmlPrinter(self.stream, self.options.get("encoding", settings.DEFAULT_CHARSET))
            self.xml.startDocument()
            self.xml.startElement("django-objects", {"version" : "1.0"})


    def end_serialization(self):
        """
        End serialization -- end the document.
        """
        if (self.root):
            self.indent(0)
            self.xml.endElement("django-objects")
            self.xml.endDocument()


    def handle_field(self, obj, field):
        """
        Called to handle each field on an object (except for ForeignKeys and
        ManyToManyFields)
        """
        self.indent(2)

        field_attrs = {
            "name" : field.name,
            "type" : field.get_internal_type()
        }
        # handle fields with a extra_attrs set as speciul
        if hasattr(field, 'extra_attrs'):
            if field.extra_attrs:
                for (key, value) in field.extra_attrs.iteritems():
                    field_attrs[key] = force_unicode(value)

            field_attrs['name'] = field.name.replace('_', '.')

        self.xml.startElement("field", field_attrs)

        # Checks for a custom value serializer 
        if not hasattr(field, '__serialize__'):
            # Get a "string version" of the object's data.
            if getattr(obj, field.name) is not None:
                self.xml.characters(field.value_to_string(obj))
            else:
                self.xml.addQuickElement("None")
        else:
            field.__serialize__(obj, self.xml)

        self.xml.endElement("field")

    def handle_fk_field(self, obj, field):
        """
        Called to handle a ForeignKey (we need to treat them slightly
        differently from regular fields).
        """
        self._start_relational_field(field)
        related = getattr(obj, field.name)
        if related is not None:
            if self.use_natural_keys and hasattr(related, 'natural_key'):
                # If related object has a natural key, use it
                related = related.natural_key()
                # Iterable natural keys are rolled out as subelements
                for key_value in related:
                    self.xml.startElement("natural", {})
                    self.xml.characters(smart_unicode(key_value))
                    self.xml.endElement("natural")
            else:
                if field.rel.field_name == related._meta.pk.name:
                    # Related to remote object via primary key
                    related = related._get_pk_val()
                else:
                    # Related to remote object via other field
                    related = getattr(related, field.rel.field_name)
                self.xml.characters(smart_unicode(related))
        else:
            self.xml.addQuickElement("None")
        self.xml.endElement("field")

    def handle_m2m_field(self, obj, field):
        """
        while easymode follows inverse relations for foreign keys,
        for manytomayfields it follows the forward relation.
        
        While easymode excludes all relations to "self" you could
        still create a loop if you add one extra level of indirection.
        """
        
        if field.rel.through._meta.auto_created:#  and obj.__class__ is not field.rel.to:
            # keep approximate recursion level
            with recursion_depth('handle_m2m_field') as recursion_level:
                
                # a stack trace is better than python crashing.
                if recursion_level > getattr(settings, 'RECURSION_LIMIT', sys.getrecursionlimit() / 10):
                    raise Exception(MANY_TO_MANY_RECURSION_LIMIT_ERROR % 
                            (field.name, obj.__class__.__name__, field.rel.to.__name__))

                self._start_relational_field(field)

                s = RecursiveXmlSerializer()
                s.serialize( getattr(obj, field.name).iterator(), xml=self.xml, stream=self.stream)

                self.xml.endElement("field")



"""
Classes to support overriding the default queryset used in django models.
These classes add a recursive __xml__ method to a django model's queryset
"""
from itertools import chain

from django.db import models
from django.db.models.query import QuerySet
from easymode.tree.xml.serializers import RecursiveXmlSerializer

class QuerySetManager(models.Manager):

    """
    Allows setting a custom QuerySet type
    """
    use_for_related_fields = True

    def __init__(self, qs_class=models.query.QuerySet):
        """Pass the type of the queryset to the constructor"""
        self.queryset_class = qs_class
        super(QuerySetManager, self).__init__()

    def get_query_set(self):
        """manager will return the custom queryset"""
        return self.queryset_class(self.model)

    def __getattr__(self, attr, *args):
        try:
            return getattr(self.__class__, attr, *args)
        except AttributeError:
            return getattr(self.get_query_set(), attr, *args)

class XmlSerializableQuerySet(QuerySet):
    """
    Adds a method to the queryset that can serialize it to xml
    """
    
    def __xml__(self):
        """turn querysets into xml recursively"""
        ser = RecursiveXmlSerializer()
        return ser.serialize(self)

class XmlQuerySetChain(list):
    """
    Can be used to combine multiple querysets and turn them into
    xml in one go.
    
    usage::
        
        from easymode.tree import xml
        from example.foobar.models import *
        
        foos = Foo.objects.all()
        bars = Bar.objects.all()
        foobars = XmlQuerySetChain(foos, bars)
        
        xml_string = xml(foobars)
    
    You can also pass an ``XmlQuerySetChain`` into ``easymode.xslt.response.render_to_response``.
    
    :param querysets: a :class:`list` of querysets that should combined into a single xml.
    """
    def __init__(self, *querysets):
        """Pass some querysets to this queryset chain and they can all be serialized together."""
        list.__init__(self, chain(*querysets))
    
    def __xml__(self):
        """turn querysets into xml recursively"""
        ser = RecursiveXmlSerializer()
        return ser.serialize(self)
    
    


from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models

from easymode.admin.models.fields import SafeTextField, SafeHTMLField
from easymode.i18n.decorators import I18n
from easymode.tree.xml.decorators import toxml


@toxml
@I18n('charfield')
class TestModel(models.Model):
    "Used in lots of test"

    charfield = models.CharField(max_length=255, unique=True)
    tags = models.ManyToManyField('TagModel', related_name='testmodels')
    
    def __unicode__(self):
        return u"%s%s - %s" % (self.__class__.__name__, self.pk, self.charfield)

    def natural_key(self):
        return (self.charfield,)
    

class TestSubModel(models.Model):
    "Used in lots of test"

    testmodel = models.ForeignKey(TestModel, related_name='submodels')
    subcharfield = models.CharField(max_length=255)
    subintegerfield = models.IntegerField()

    def __unicode__(self):
        return u"%s%s - %s" % (self.__class__.__name__, self.pk, self.subcharfield)


class TestSecondSubmodel(models.Model):
    """hangs on to TestModel as well"""
    
    testmodel = models.ForeignKey(TestModel, related_name='secondsubmodels')
    ultrafield = models.TextField()
    
    def __unicode__(self):
        return self.ultrafield


class TestSubSubModel(models.Model):
    "Used in lots of test"
    
    testsubmodel = models.ForeignKey(TestSubModel, related_name="subsubmodels")
    subsubcharfield = models.CharField(max_length=255)
    
    def __unicode__(self):
        return self.subsubcharfield


@I18n('value')
class TagModel(models.Model):
    "Used in test_generic_relations_also_work (easymode.tests.testcases.testtoxml.RecursiveSerializerTest)"
    value = models.CharField(max_length=23)

    def __serialize__(self, stream):
        stream.startElement('taggy', {})
        stream.characters(self.value)
        stream.endElement('taggy')

@toxml
@I18n('title', 'description')
class TestL10nModel(models.Model):
    """
    Used in easymode.tests.testcases.testi18n.Testi18n and
    easymode.tests.testcases.testsafefields.TestSafeFields.
    """
    title = models.CharField(max_length=200)
    description = SafeTextField()
    body = SafeHTMLField(default='hi i am an error')
    price = models.FloatField()

    def __unicode__(self):
        return u"%s%s" % (self.__class__.__name__, self.pk)


class GenericRelatedModel(models.Model):
    "Used in test_generic_relations_also_work (easymode.tests.testcases.testtoxml.RecursiveSerializerTest)"

    content_type = models.ForeignKey(ContentType)
    parent_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'parent_id')

    name = models.CharField(max_length=100)

    def __unicode__(self):
        return u"%s%s" % (self.__class__.__name__, self.pk)


@toxml
class TestGenericFkModel(models.Model):
    "Used in used in test_generic_relations_also_work (easymode.tests.testcases.testtoxml.RecursiveSerializerTest)"

    name = models.CharField(max_length=100)
    relateds = generic.GenericRelation(GenericRelatedModel, content_type_field='content_type', object_id_field='parent_id')

    def __unicode__(self):
        return u"%s%s" % (self.__class__.__name__, self.pk)


@I18n('a', 'b')
class ManagerErrorModel(models.Model):
    "used in easymode.tests.testcases.testintrospection.TestIntrospection"
    a = models.CharField(max_length=255)
    b = models.ImageField(upload_to='uploads')
    c = models.TextField()

    def __unicode__(self):
        return u"%s %s %s %s" % (self.pk, self.a, self.b, self.c)

@I18n('number', 'another')
class FormTestModel(models.Model):
    "Used in test_form_is_overridden (easymode.tests.testcases.testforms.TestL10nForm"
    number = models.IntegerField()
    other = models.CharField(max_length=255)
    another = models.CharField(max_length=255)


class TopModel(models.Model):
    title = models.CharField('The title', max_length=255)
    subtitle = SafeTextField('The subtitle')
    
    def __unicode__(self):
        return self.title
    
class BottomModel(models.Model):
    top = models.ForeignKey(TopModel)
    
    comment = SafeTextField('Some comment')
    
    def __unicode__(self):
        return self.comment
    



from django.contrib import admin
from django.contrib.admin import AdminSite
from django.contrib.auth import login
from django.contrib.auth.models import User
from django.core.handlers.base import BaseHandler
from django.db import models
from django.test import TestCase, RequestFactory

from easymode.i18n.admin.decorators import L10n
from easymode.i18n.decorators import I18n


__all__ = ('EasyModeReadOnlyFieldsTest',)


class RequestMock(RequestFactory):
    """
    Construct a generic request object, with session support. """
    def request(self, **request):
        request = RequestFactory.request(self, **request)
        handler = BaseHandler()
        handler.load_middleware()
        for middleware_method in handler._request_middleware:
            if middleware_method(request):
                raise Exception("Couldn't create request mock object - "
                                "request middleware returned a response")
        return request


@I18n('computer', 'says', 'no')
class ComputerSaysNo(models.Model):
    """ Demo model. No-field is readonly in admin. """
    computer = models.CharField(max_length=255)
    says = models.CharField(max_length=255)
    no = models.CharField(max_length=255)

    class Meta:
        ordering = ('id',)

@L10n
class ComputerSaysNoAdmin(admin.ModelAdmin):
    """ Has one readonly field. """
    model = ComputerSaysNo
    readonly_fields = ('no',)


class EasyModeReadOnlyFieldsTest(TestCase):

    def setUp(self):
        self.site = AdminSite()
        self.request = None
        self.factory = RequestMock()
        self.u1 = User.objects.create_superuser(
            username='harry',
            email='bla',
            password='bla'
        )
        self.u1.backend = 'django.contrib.auth.backends.ModelBackend'

    def test_form_submit_submit_readonly(self):
        """
        Tests if form submit works with readonly fields.
        """
        ma = ComputerSaysNoAdmin(ComputerSaysNo, self.site)
        request = self.factory.get('/admin/foobar/')
        login(request, self.u1)
        form = ma.get_form(request)
        f = form({'computer': 'hallo', 'says': 'hello'})
        self.assertTrue(f.is_valid(), str(f.errors.as_text()))

    def test_form_submit_submit_no_readonly(self):
        """
        Tests if form submit works when no readonly fields are present
        """
        ma = ComputerSaysNoAdmin(ComputerSaysNo, self.site)

        request = self.factory.get('/admin/foobar/')
        login(request, self.u1)
        form = ma.get_form(request)
        f = form({'computer': 'hallo', 'says': 'hello', 'no': 'hello'})
        self.assertTrue(f.is_valid(), str(f.errors.as_text()))



"""
When ``get_absolute_url`` is defined on a model, django will place a
*view on site* link in the ``change_view`` of the model. 

If you are using easymode's multilanguage features and especially 
:class:`~easymode.middleware.LocaleFromUrlMiddleWare` and
:class:`~easymode.middleware.LocaliseUrlsMiddleware` you will find that the
*view on site* link will not point to the correct (current) language.

The routes specified here will override the default admin routes and
make the *view on site* link take the current language into account.

include the urls in your urlconf like this::

    (r'^', include('easymode.urls')),

"""
from django.conf.urls.defaults import *

from easymode.utils.languagecode import get_language_codes_as_disjunction

languages = get_language_codes_as_disjunction()

urlpatterns = patterns('easymode.views',
    (r'^admin/r/(?P<content_type_id>\d+)/(?P<object_id>.+)/$', 'preview'),
    (r'^(%(languages)s)/admin/r/(?P<content_type_id>\d+)/(?P<object_id>.+)/$' % locals(), 'preview'),
)



from django.http import HttpResponse
from django.template.loader import find_template_source
from django.test import TestCase

from easymode.tree import xml as tree
from easymode.tests.models import TestModel
from easymode.tests.testcases import initdb
from easymode.utils.languagecode import get_language_codes
from easymode.xslt import response


if 'en-us' not in get_language_codes():
    raise Exception('the language "en-us" must be in your LANGUAGES to run the test suite')

__all__ = ('XsltTest',)

@initdb
class XsltTest(TestCase):
    """
    Test the functionality related to xslt
    """
    
    def setUp(self):
        t = TestModel(charfield='Hoi Ik ben de root node')
        t.save()
        f = t.submodels.create(subcharfield="Hoi Ik ben de first level sub node", subintegerfield=10)
        s = t.submodels.create(subcharfield="Hoi ik ben de tweede first level sub node", subintegerfield=100)
        w = t.secondsubmodels.create(ultrafield="Sed tempor. Ut felis. Maecenas erat.")
        f.subsubmodels.create(subsubcharfield="Hoi ik ben de thord level sub node")
        f.subsubmodels.create(subsubcharfield="Hoi ik ben de third level sub node")
        s.subsubmodels.create(subsubcharfield="Hoi ik ben de third level sub node")
        s.subsubmodels.create(subsubcharfield="Hoi ik ben de third level sub node")        
        t.save()
    
    def test_xsl_template_can_be_found(self):
        """docstring for test_xsl_template_can_be_found"""
        template = find_template_source('xslt/model-to-xml.xsl')
        assert(template)

    def test_xslt_template_will_render_one_object(self):
        """docstring for test_xslt_template_will_render"""
        data = TestModel.objects.get(pk=1)
        resp = response.render_to_response('xslt/model-to-xml.xsl', data)
        assert(resp)

    def test_xslt_changes_the_xml(self):
        """docstring for test_xslt_changes_the_xml"""
        data = TestModel.objects.get(pk=1)
        resp = response.render_to_response('xslt/model-to-xml.xsl', data)
        assert(resp != HttpResponse(tree.xml(data)))

    def test_xslt_will_render_queryset(self):
        """docstring for test_xslt_will_render_queryset"""
        data = TestModel.objects.all()
        resp = response.render_to_response('xslt/model-to-xml.xsl', data)
        assert(resp)


"""
Defines a :func:`render_to_response` function that resembles django.utils's
:func:`~django.shortcuts.render_to_response` helper. Only this time the template is an xslt and the object
passed as the second argument must have a ``__xml__`` method. (See :func:`~easymode.tree.xml.decorators.toxml`)
"""
from django.http import HttpResponse

from easymode.tree import xml as xmltree
from easymode.utils.template import find_template_path
from easymode.xslt import transform


def render_to_response(template, object, params=None, mimetype='text/html'):
    """
    ``object`` will be converted to xml using :func:`easymode.tree.xml`. The resulting xml 
    will be transformed using ``template``.
    The result will be a :class:`~django.http.HttpResponse` object,
    containing the transformed xml as the body.
    
    :param template: an xslt template name.
    :param object: an object that has an ``__xml__`` method. (See :func:`easymode.tree.xml.decorators.toxml`).
    :param params: A dictionary containing xslt parameters. Use :func:`~easymode.xslt.prepare_string_param`\
        on strings you want to pass in.
    :param mimetype: The mimetype of the :class:`~django.http.HttpResponse`
    :rtype: :class:`django.http.HttpResponse`
    """
    xsl_path = find_template_path(template)
    xml = xmltree.xml(object)
    
    result = transform(xml, str(xsl_path), params)
    return HttpResponse(result, mimetype=mimetype)
    
def render_to_string(template, object, params=None):
    """
    ``object`` will be converted to xml using :func:`easymode.tree.xml`. The resulting xml 
    will be transformed using ``template``.
    The result is a unicode string containing the transformed xml.
    
    :param template: an xslt template name.
    :param object: an object that has an ``__xml__`` method. (See :func:`easymode.tree.xml.decorators.toxml`).
    :param params: A dictionary containing xslt parameters. Use :func:`~easymode.xslt.prepare_string_param`\
        on strings you want to pass in.
    :rtype: :class:`unicode`
    """
    xsl_path = find_template_path(template)
    xml = xmltree.xml(object)
    
    result = transform(xml, str(xsl_path), params)
    return result

def render_xml_to_string(template, input, params=None):
    """
    Transforms ``input`` using ``template``, which should be an xslt.
    
    :param template: an xslt template name.
    :param input: an string that contains xml
    :param params: A dictionary containing xslt parameters. Use :func:`~easymode.xslt.prepare_string_param`\
        on strings you want to pass in.
    :rtype: :class:`unicode`
    """
    xsl_path = find_template_path(template)

    result = transform(input, str(xsl_path), params)  
    return result



"""
Contains xslt transformation functionality to be used with django models
"""

__all__ = ('XsltError', 'transform', 'prepare_string_param', 'response')


class XsltError(Exception):
    """The xslt transformation resulted in an error"""
    pass


def _transform_libxslt(xml, xslt, params=None):
    """
    Transform ``xslt`` using any of the 3 supported xslt engines:

    - `lxml <http://codespeak.net/lxml/>`_
    - `libxml <http://xmlsoft.org/python.html>`_

    :param xml: The xml to be transformed.
    :param xslt: The xslt to be used when transforming the ``xml``.
    :param params: A dictionary containing xslt parameters. Use :func:`~easymode.xslt.prepare_string_param`\
        on strings you want to pass in.
    """
    try:
        xslt_doc = libxml2.parseFile(xslt)
        xslt_proc = libxslt.parseStylesheetDoc(xslt_doc)
        xml_doc = libxml2.parseDoc(xml)
        
        result = xslt_proc.applyStylesheet(xml_doc, params)

        xml_string = str(result)
        xml_unicode_string = xml_string.decode('utf-8')

        xslt_proc.freeStylesheet()
        xml_doc.freeDoc()

        return xml_unicode_string
    except RuntimeError as e:
        raise XsltError(str(e))


def _transform_lxml(xml, xslt_path, params=None):
    """
    Transform ``xslt`` using any of the 2 supported xslt engines:

    - `lxml <http://codespeak.net/lxml/>`_
    - `libxml <http://xmlsoft.org/python.html>`_

    :param xml: The xml to be transformed.
    :param xslt: The xslt to be used when transforming the ``xml``.
    :param params: A dictionary containing xslt parameters. Use :func:`~easymode.xslt.prepare_string_param`\
        on strings you want to pass in.
    """
    # assuming params where created with prepare_string_param,
    # they are encoded as unicode, which lxml does not like
    
    xml_doc = etree.fromstring(xml)
    xslt_doc = etree.parse(xslt_path)
    xslt_proc = etree.XSLT(xslt_doc)

    if params:
        for (key, value) in params.iteritems():
            params[key] = value.decode('utf-8')
        result = xslt_proc(xml_doc, **params)
    else:
        result = xslt_proc(xml_doc)
    
    return unicode(result)

# determine which xslt engine to use,
try:
    from lxml import etree

    transform = _transform_lxml
except:
    import libxslt
    import libxml2

    transform = _transform_libxslt


def prepare_string_param(string):
    """
    Prepare a string for passing as a parameter to the xslt processor.
    
    :param string: The string you want to pass to the xslt.
    """
    result = u"'%s'" % (string.replace("'","&apos;") or '')
    return result.encode('utf-8')



from django.contrib import admin
from django.http import HttpResponseRedirect

from easymode.tree.admin.forms import RecursiveInlineFormSet
from easymode.tree.admin.relation import _CanFindParentLink

__all__ = ('LinkInline', 'LinkedItemAdmin')

class LinkInline(admin.StackedInline):
    """
    A base class for an admin inline that will render as links to the change_view.
    This means you have to also register the model you are inlining with a non-inline
    ModelAdmin::
        
        class BottomAdmin(InvisibleModelAdmin):
            parent_link = 'top'
        
        class BottomLinkInline(LinkInline):
            fields = ('top',)
            model = BottomModel
    
    .. attribute:: fields
        
        It is mandatory to set the fields or fieldsets attribute of a class extending
        ``LinkInline``. If you don't, all the fields will be show, instead of only
        the change or add button. Ofcourse it can't hurt to add some ``read_only_fields``
        to add some context.
    """
    formset = RecursiveInlineFormSet
    template = 'tree/admin/edit_inline/link_inline.html'
    extra = 1

class LinkedItemAdmin(admin.ModelAdmin, _CanFindParentLink):
    """
    This class should be used as the base class for all admin classes that have
    :class:`~easymode.tree.admin.abstract.LinkInline` inlines::
    
        class TopAdmin(LinkedItemAdmin):
            inlines = [BottomLinkInline]
        
    Also it can be used
    to make save return to a parent model in a tree context. You could also use
    :class:`~easymode.tree.admin.relation.InvisibleModelAdmin`, and then it would
    also not be shown in the admin app list.
    
    .. attribute:: parent_link
        
        To have sane breadcrumbs if the :class:`~easymode.tree.admin.abstract.LinkedItemAdmin`
        is used as child of another :class:`~easymode.tree.admin.abstract.LinkedItemAdmin`
        and make the save button return to the parent instead of the app listing, the
        ``parent_link`` should be set.
        
        It must be set to the *name* of the ``ForeignKey`` that points to the parent.
    
    """
    change_form_template = 'tree/admin/change_form_with_parent_link.html'
    
    def change_view(self, request, object_id, extra_context=None):
        
        # retrieve link to parent for breadcrumb path
        defaults = self._get_parent_link(object_id)
        
        if extra_context:
            defaults.update(extra_context)
        
        response = super(LinkedItemAdmin, self).change_view(request, object_id, defaults)
        
        if response.get('Location', False) == '../':
            return HttpResponseRedirect(defaults.get('parent_model', '../'))
        
        return response
    
    class Media:
        js = (
            'easymode/js/adminoverride.js',
        )
    
    


"""
Internationalization and localization support for django models.
"""

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

if not hasattr(settings, 'PROJECT_DIR'):
    raise ImproperlyConfigured(
        """
        PLEASE DEFINE PROJECT_DIR AS: 
        PROJECT_DIR = os.path.dirname(__file__) 
        in your settings.py.
        
        Otherwise you can not use easymode.i18n.
        """)

from weakref import WeakKeyDictionary

from django.db.models.signals import post_save

import easymode.i18n.gettext

__all__ = ('register', 'unregister', 'admin', 'decorators', 'gettext', 'meta')

_post_save_handlers = WeakKeyDictionary()

def register(cls, location=None):
    """
    Register a model for translation
    like so:
    
    >>> from easymode import i18n
    >>> i18n.register(SomeModel)
    
    This will add entries to the gettext catalog located in the
    ``settings.LOCALE_DIR`` or directory.
    
    You can also store the catalogs somewhere else. The next example stores the
    catalog in the same dir as the file in which i18n.register is called:
    
    >>> i18n.register(SomeModel, __file__)
    
    You can also explictly give a path:
    
    >>> i18n.register(SomeModel, '/tmp/)
    
    Please note that the entire locale is stored in that path, so if the path
    is /tmp/ the locale is: /tmp/locale/ this directory will be created if it
    does not exist.
    
    :param cls: The model class that should be translated by means of gettext.
    :param location: The location of the po file. This can be either a directory\
        or a file name. If left unspecified, ``settings.LOCALE_DIR`` or\
        ``settings.PROJECT_DIR`` will be used.
    """
    
    create_po_for_model = easymode.i18n.gettext.MakeModelMessages(location, cls)
    
    _post_save_handlers[cls] = create_po_for_model
    
    post_save.connect(create_po_for_model, cls, False)

def unregister(cls):
    """
    unregisters a previously registered model.
    
    This means the po file will not be updated when the model is saved.
    
    :param cls: The model class that should nolonger be translated by means of gettext.
    """
    handler = _post_save_handlers.get(cls, None)
    if handler:
        post_save.disconnect(handler, cls)


"""
Derived from django-transmeta.
Contains machinery to change a type after construction,
by using a class decorator.

This in contrast with django-transmeta which requires
overriding the __metaclass__.
"""
import copy

from django.conf import settings
from django.contrib.admin.options import FORMFIELD_FOR_DBFIELD_DEFAULTS
from django.db.models.fields import NOT_PROVIDED
from django.utils import translation

from easymode.utils.languagecode import get_all_language_codes, \
    get_real_fieldname
from easymode.i18n.meta.fields import DefaultFieldDescriptor
from easymode.i18n.meta.utils import get_field_from_model_by_name


__all__ = ('localize_fields',)


def localize_fields(cls, localized_fields):
    """
    For each field name in localized_fields,
    for each language in settings.LANGUAGES,
    add fields to cls,
    and remove the original field, instead
    replace it with a DefaultFieldDescriptor,
    which always returns the field in the current language.
    """

    # never do this twice
    if hasattr(cls, 'localized_fields'):
        return cls

    # MSGID_LANGUAGE is the language that is used for the gettext message id's.
    # If it is not available, because the site isn't using subsites, the
    # LANGUAGE_CODE is good too. MSGID_LANGUAGE gives the opportunity to
    # specify a language not available in the site but which is still used for
    # the message id's.
    msgid_language = getattr(settings,
        'MSGID_LANGUAGE',  settings.LANGUAGE_CODE)

    # set the localized fields property
    cls.localized_fields = localized_fields

    for field in localized_fields:
        original_attr = get_field_from_model_by_name(cls, field)

        for cnt, language_code in enumerate(get_all_language_codes()):
            i18n_attr = copy.copy(original_attr)
            # add support for south introspection.
            i18n_attr._south_introspects = True
            i18n_attr.original_fieldname = field
            i18n_attr.include_in_xml = False
            lang_attr_name = get_real_fieldname(field, language_code)
            i18n_attr.name = lang_attr_name
            i18n_attr.creation_counter = i18n_attr.creation_counter + .01 * cnt
            # null must be allowed for the message id language because this
            # language might not be available at all in the backend
            if not i18n_attr.null and i18n_attr.default is NOT_PROVIDED:
                i18n_attr.null = True

            if language_code != msgid_language:
                # no validation for the fields that are language specific
                if not i18n_attr.blank:
                    i18n_attr.blank = True

            if i18n_attr.verbose_name:
                i18n_attr.verbose_name = translation.string_concat(
                    i18n_attr.verbose_name, u' (%s)' % language_code)
            cls.add_to_class(lang_attr_name, i18n_attr)

        # delete original field
        del cls._meta.local_fields[cls._meta.local_fields.index(original_attr)]

        # copy some values and functions from the original_attr
        # so the field can emulate the original_attr as good as possible
        kwargs = {
            'serialize': getattr(original_attr, 'serialize', True),
            'extra_attrs': getattr(original_attr, 'extra_attrs', None),
            'max_length': getattr(original_attr, 'max_length', None),
            'min_length': getattr(original_attr, 'min_length', None),
            'form_field': original_attr.formfield(
                **FORMFIELD_FOR_DBFIELD_DEFAULTS.get(
                    original_attr.__class__, {})),
            'get_internal_type': original_attr.get_internal_type,
            'unique': getattr(original_attr, 'unique', False),
            'to_python': original_attr.to_python,
        }

        # copy __serialize__ if it was defined on the original attr
        if hasattr(original_attr, '__serialize__'):
            kwargs['__serialize__'] = original_attr.__serialize__

        # add the DefaultFieldDescriptor where the original_attr was.
        cls.add_to_class(field, DefaultFieldDescriptor(field, **kwargs))

        # update fields cache
        cls._meta._fill_fields_cache()

    # return the finished product
    return cls



from django.contrib import admin

from easymode.i18n.admin.decorators import L10n
from easymode.tests import models as test_models, forms as test_forms
from easymode.tree.admin import relation
from easymode.tree.admin.relation import InvisibleModelAdmin
from easymode.tree.admin.abstract import LinkedItemAdmin, LinkInline
from easymode.tests.models import BottomModel, TopModel


class TestSubModelInline(admin.StackedInline):
    """docstring for TestSubModelInline"""
    model = test_models.TestSubModel
    extra = 1
    order_field = 'order'


class TestModelAdmin(relation.ForeignKeyAwareModelAdmin):
    """docstring for TestModelAdmin"""

    inlines = [TestSubModelInline]
    
    children = [test_models.TestSubModel,test_models.TestSecondSubmodel]
    invisible_in_admin = False


class SelfAwareTestModelAdmin(relation.ForeignKeyAwareModelAdmin):
    model = test_models.TestModel


@L10n(test_models.TestL10nModel)
class TestL10nModelAdmin(admin.ModelAdmin):
    pass


@L10n
class FormTestAdmin(admin.ModelAdmin):
    form = test_forms.OverrideForm
    model = test_models.FormTestModel


@L10n
class ManagerErrorModelAdmin(admin.ModelAdmin):
    model = test_models.ManagerErrorModel
    
    def delete_view(self, request, object_id, extra_context=None):
        return super(ManagerErrorModelAdmin, self).delete_view(request, object_id, extra_context)


class BottomAdmin(InvisibleModelAdmin):
    """
    I am using InvisibleModelAdmin as a base class here so I can get the
    parent_link functionality and also that BottomAdmin is not visible in
    the admin listing (see old tree for more info). I could've used
    LinkedItemAdmin as well, if I would've been interested in parent_link
    only. This is the change view for the inlined item.
    """
    parent_link = 'top'


class BottomLinkInline(LinkInline):
    """
    This is the inline view of the inlined item. It will de rendered as a
    link to the change view or add view of the inlined item.
    
    NOTE that you MUST define fields, which must only include the foreign key.
    Ofcourse you might want to include some more fields and put them as
    read_only_fields, to give a bit more info.
    """
    fields = ('top',)
    model = BottomModel


class TopAdmin(LinkedItemAdmin):
    """
    This is the top view that has the inlines. It has a special form and
    template to make it display the foreign key fields as links.
    """
    inlines = [BottomLinkInline]

admin.site.register(test_models.TestModel, TestModelAdmin)
admin.site.register(test_models.TestSubModel)
admin.site.register(test_models.TestSecondSubmodel)
admin.site.register(test_models.TestL10nModel, TestL10nModelAdmin)
admin.site.register(test_models.ManagerErrorModel, ManagerErrorModelAdmin)
admin.site.register(test_models.FormTestModel, FormTestAdmin)
admin.site.register(TopModel, TopAdmin)
admin.site.register(BottomModel, BottomAdmin)



from django.core.urlresolvers import reverse
from django.test import TestCase
from django.utils.translation import get_language, activate

from easymode.tests.testcases import initdb
from easymode.utils.languagecode import fix_language_code


ADD_BUTTON = """<a href="%s" class="add-another" onclick="return showAddAnotherPopup(this);">
    <img src="/admin/media/img/admin/icon_addlink.gif" width="10" height="10" """

__all__ = ('TestRelatedAdmin',)

@initdb
class TestRelatedAdmin(TestCase):
    fixtures = ['auth-user', 'tests-top-bottom']
    
    def test_the_tree_templates_get_called(self):
        "the tree templates should be called for models with tree admins"
        
        if self.client.login(username='admin', password='admin'):
            url = reverse('admin:tests_topmodel_change', args=[1])
            response = self.client.get(url)

            self.assertTemplateUsed(response, 'tree/admin/change_form_with_parent_link.html')
            self.assertTemplateUsed(response,'tree/admin/edit_inline/link_inline.html')
            self.assertTemplateUsed(response, 'tree/admin/widgets/add_widget.html')
            self.assertTemplateUsed(response, 'tree/admin/widgets/link_widget.html')
        else:
            self.fail('Can not login with admin:admin')
    
    def test_adding_new_inline_adds_it_in_the_top_model_view(self):
        "adding a bottom model should make it appear in inlines"
        
        if self.client.login(username='admin', password='admin'):
            top_url = reverse('admin:tests_topmodel_change', args=[1])
            response = self.client.get(top_url)
            self.assertEqual(response.status_code, 200)
            self.assertNotContains(response, 'Chicken is good')
            
            bottom_url = reverse('admin:tests_bottommodel_add')
            data = {
                'top':1,
                'comment':'Chicken is good',
            }
            response = self.client.post(bottom_url, data)
            self.assertEqual(response.status_code, 302)
            
            response = self.client.get(top_url)
            self.assertContains(response, 'Chicken is good')
        else:
            self.fail('Can not log in with admin:admin')
    
    def test_the_plus_button_goes_to_the_correct_add_view(self):
        "There should be a plus button for the inline group that goes to the correct add view"
        if self.client.login(username='admin', password='admin'):
            top_url = reverse('admin:tests_topmodel_change', args=[1])
            response = self.client.get(top_url)
            self.assertEqual(response.status_code, 200)
            
            #/en/admin/tests/bottommodel/add/?top=1
            add_url = "%s?top=1" % reverse('admin:tests_bottommodel_add')
            localized_add_url = fix_language_code(add_url, get_language())
            
            self.assertContains(response, ADD_BUTTON % localized_add_url)
            
            # test the same for a different locale
            activate('de')
            self.assertEqual('de', get_language())
            localized_add_url = fix_language_code(add_url, get_language())
            localized_top_url = fix_language_code(top_url, get_language())
            response = self.client.get(localized_top_url)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, ADD_BUTTON % localized_add_url)
            
        else:
            self.fail('Can not log in with admin:admin')

    def test_add_button_is_not_shown_in_add_view(self):
        "The add button must be hidden in the add view because you can't add nothing to an object that does not exist yet"
        if self.client.login(username='admin', password='admin'):
            top_url = reverse('admin:tests_topmodel_add')
            response = self.client.get(top_url)
            
            add_url = "%s?top=None" % reverse('admin:tests_bottommodel_add')
            localized_add_url = fix_language_code(add_url, get_language())
            
            self.assertNotContains(response, ADD_BUTTON % localized_add_url)


from django.core.urlresolvers import reverse
from django.forms import IntegerField
from django.forms.models import BaseInlineFormSet
from django.utils.encoding import force_unicode

from easymode.tree.admin.widgets.foreignkey import LinkWidget


class RecursiveInlineFormSet(BaseInlineFormSet):
    """
    A formset that can be used to make the primary key field visible in an
    InlineModelAdmin form.
    """
    
    PRIMARY_KEY_WIDGET = LinkWidget
    
    def add_fields(self, form, index):
        super(RecursiveInlineFormSet, self).add_fields(form, index)
        
        # get the primary key and it's options
        hidden_pk_field = form.fields[self._pk_field.name]
        options =  hidden_pk_field.queryset.model._meta
        
        # the only fields that must show is the primary key
        form._meta.fields = [self._pk_field.name]
        
        widget_attrs = {}
        
        # show the link widget if the item allready exists
        if hidden_pk_field.initial is not None: 
            widget_attrs['url'] = reverse('admin:%s_%s_change' % (options.app_label, options.object_name.lower()), args=[hidden_pk_field.initial])
            widget_attrs['label'] = 'Edit %s' % force_unicode(options.verbose_name)
            
        # if the parent model exists but the item doesn't show the add button
        elif self.instance.pk is not None:
            url = reverse('admin:%s_%s_add' % (options.app_label, options.object_name.lower()))
            widget_attrs['url'] = '%s?%s=%s' % (url, self.fk.name, self.instance.pk)
            widget_attrs['label'] = 'Add %s' % force_unicode(options.verbose_name)
            widget_attrs['popup'] = True
            
        # if the parent model does not exist use an empty link widget.
        else:
            widget_attrs['url'] = None
            widget_attrs['label'] = ''
        
        # turn the primary key into a field that displays a fieldset with a link
        form.fields[self._pk_field.name] = IntegerField(hidden_pk_field.queryset, 
            initial=hidden_pk_field.initial, required=False,
            widget=self.PRIMARY_KEY_WIDGET(attrs=widget_attrs)
        )



from django.core.urlresolvers import reverse
from django.test import TestCase

from easymode.tests.models import TestL10nModel
from easymode.tests.testcases import initdb


__all__ = ('TestSafeFields',)

@initdb
class TestSafeFields(TestCase):
    "Test for fields that strip cariage returns."
    
    fixtures = ['auth-user', 'tests-testl10nmodel']
    
    def test_safe_fields_strip_cariage_returns(self):
        "Safe fields should strip cariage returns from the input"
                
        h = {
            'title': "Ik ben de groot moeftie van oeral",
            'description': "Ik weet\r weinig tot niks van brei \rcursussen",
            'body':"Het zou kunnen dat\r linksaf\r slaan hier een goede keuze is.",
            'price':34.0,
        }
        
        if self.client.login(username='admin', password='admin'):
            
            add_view_url = reverse("admin:tests_testl10nmodel_add")            
            response = self.client.post(add_view_url, h)

            i = TestL10nModel.objects.get(pk=3)

            assert(i.title == 'Ik ben de groot moeftie van oeral')
            assert(i.description == "Ik weet weinig tot niks van brei cursussen")
            assert(i.body == "Het zou kunnen dat linksaf slaan hier een goede keuze is.")
        else:
            self.fail("can not login to backend using admin/admin")



"""
Fields to be used in models.
"""
import logging
import os
import re
import warnings
from StringIO import StringIO
from time import time
from urllib import urlopen
from xml import sax

from django.conf import settings
from django.contrib.admin.widgets import AdminTextInputWidget, AdminTextareaWidget
from django.db import connection, models
from django.db.models import CharField, TextField
from django.forms import fields
from django.utils.datastructures import DictWrapper
from django.utils.translation import ugettext_lazy as _
from tinymce.widgets import TinyMCE

from easymode.admin.forms import fields as form_fields
from easymode.utils import xmlutils


try:
    import tinymce
except ImportError:
    warnings.warn(
        """
        Without tinymce most of the fields related to rich text do not work, 
        this will also cause some of tests to fail!!
        http://code.google.com/p/django-tinymce/
        """
    )



__all__ = ('FlashUrlField', 'RelativeFilePathField', 'XmlField', 'SafeTextField', 'SafeHTMLField'
)

DEFAULT_MCE_CONFIG = None
if hasattr(settings, 'TINYMCE_DEFAULT_CONFIG'):
    DEFAULT_MCE_CONFIG = settings.TINYMCE_DEFAULT_CONFIG.get('theme_advanced_buttons1')

class FlashUrlField(CharField):
    """
    A field that is an url in flash.
    
    A valid flash url must be one of:
    
    1. an absolute url, eg. *http://example.com/*
    2. a relative url, eg. */main/chapter* 
    3. a flash url with a hash eg. *#/main/chapter*
    """
    
    _south_introspects = True

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.pop('max_length', 512)
        super(FlashUrlField, self).__init__(*args, **kwargs)
    
    def get_internal_type(self):
        return self.__class__.__name__

    def db_type(self, connection):
        data = DictWrapper(self.__dict__, connection.ops.quote_name, "qn_")
        try:
            return connection.creation.data_types['CharField'] % data
        except KeyError:
            return None

    def formfield(self, **kwargs):
        defaults = {
            'form_class': form_fields.FlashUrlField,
            'max_length': self.max_length or 512,
            'error_messages': {
                'invalid': _('Alleen valide urls of relative flash urls die beginnen met #/ zijn geoorloofd')
            }
        }
        defaults.update(kwargs)
        return super(FlashUrlField, self).formfield(**defaults)

class SafeTextField(models.TextField):
    """
    Removes all cariage returns from the input.

    This class should be used instead of :class:`django.db.models.TextField`, if you
    want to use easymode's gettext facilities. It will strip all cariage returns from
    the input. Cariage returns are an error when included in gettext message id's. It
    also allows you to specify the number of rows and columns of the textarea.
    
    :param rows: The number of rows of the textarea.
    :param cols: The number of columns of the textarea.
    """

    _south_introspects = True
    
    def __init__(self, *args, **kwargs):
        self.rows = kwargs.pop('rows', 5)
        self.cols = kwargs.pop('cols', 40)
        
        super(SafeTextField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'widget': AdminTextareaWidget(attrs={'rows':self.rows, 'cols':self.cols}),
            'form_class': form_fields.PoSafeTextField
        }
        defaults.update(kwargs)
        return super(SafeTextField, self).formfield(**defaults)


class SafeHTMLField(TextField):
    """
    A html field with the following properties:

    1. Escapes cariage returns
    2. Will render as html, without escaping &lt; and &gt;
    3. Validates that html and will log errors when invalid html 
       is discovered, instead of crash.
    4. Allows the max_length attribute, the max_length is then computed,
       without counting markup. Also html entities are converted to normal
       characters before measuring the length as well.
    5. accepts tinymce configuration options.

    usage::

        html = SafeHTMLField(_('some rich text'), max_length=200, width=300, height=100, buttons="link,unlink,code")

    :param max_length: The maximum length of the text, after stripping markup and
        converting html entities to normal characters.
    :param width: The width of the html editor in the admin
    :param height: The height of the html editor in the admin
    :param buttons: Configuration for the 
        `theme_advanced_buttons1 <http://wiki.moxiecode.com/index.php/TinyMCE:Configuration/theme_advanced_buttons_1_n>`_. 
        The default is "bullist,numlist,|,undo,redo,|,link,unlink,|,code,|,cleanup,removeformat".
    """
    _south_introspects = True

    def __init__(self, *args, **kwargs):
        mce_config = DEFAULT_MCE_CONFIG or \
            "bullist,numlist,|,undo,redo,|,link,unlink,|,code,|,cleanup,removeformat"
        
        self.mce_width = kwargs.pop('width', 340)
        self.mce_height = kwargs.pop('height', 160)
        self.buttons = kwargs.pop('buttons', mce_config)
        
        super(SafeHTMLField, self).__init__(*args, **kwargs)

    def __serialize__(self, obj, xml):
        richtext = self.value_to_string(obj)
        value = u"<richtext>%s</richtext>"  % richtext

        if xmlutils.is_valid(value):
            parser = sax.make_parser(["easymode.utils.xmlutils"])
            parser.setContentHandler(xml)
            parser.parse(StringIO(value.encode('utf-8')))            
        else:
            logging.error('Invalid xml in %s: %s' % (obj, value))

    def formfield(self, **kwargs):
        mce_default_attrs = {
            'theme_advanced_buttons1' : self.buttons,
            'theme_advanced_resizing' : True,
            'width': self.mce_width,
            'height': self. mce_height,
        }

        defaults = {
            'form_class': form_fields.PoSafeTextField,
        }

        # if this thing has a max_length then use a 
        # special form field
        if hasattr(self, 'max_length'):
            defaults['form_class'] = form_fields.HtmlEntityField
            defaults['max_length'] = self.max_length

        defaults.update(kwargs)
        defaults['widget'] = TinyMCE(mce_attrs=mce_default_attrs)
        
        return super(SafeHTMLField, self).formfield(**defaults)


class RelativeFilePathField(fields.ChoiceField):
    """
    A FilePathField which stores paths as relative paths.
    
    :param base: The base path, this base path will **NOT** be in the database (eg. settings.PROJECT_DIR).
    :param relative_path: The relative part of the path, which will be in the database, together with the filename.
    """
    
    _south_introspects = True
    
    def __init__(self, base, relative_path, match=None, recursive=False, required=True,
                 widget=None, label=None, initial=None, help_text=None, path='',
                 *args, **kwargs):
        self.base, self.relative_path, self.match, self.recursive = base, relative_path, match, recursive

        super(RelativeFilePathField, self).__init__(choices=(), required=required,
            widget=widget, label=label, initial=initial, help_text=help_text,
            *args, **kwargs)

        if self.required:
            self.choices = []
        else:
            self.choices = [("", "---------")]

        if self.match is not None:
            self.match_re = re.compile(self.match)

        if recursive:
            for root, dirs, files in os.walk(self.path):
                for f in files:
                    if self.match is None or self.match_re.search(f):
                        f = os.path.join(root, f)
                        self.choices.append((f.replace(self.base, ""), f.replace(self.path, "", 1)))
        else:
            try:
                for f in os.listdir(self.path):
                    full_file = os.path.join(self.path, f)
                    if os.path.isfile(full_file) and (self.match is None or self.match_re.search(f)):
                        self.choices.append((os.path.join(self.relative_path, f), f))
            except OSError:
                pass

        self.widget.choices = self.choices

    @property
    def path(self):
        return os.path.join(self.base, self.relative_path)


class XmlField(models.TextField):
    """
    Used to store XML.
    
    When using :func:`easymode.tree.xml`, all data is escaped and
    html tags are turned into entities. Using this field will make
    sure the escaping is skipped, so xml is rendered unmodified.
    """
    
    _south_introspects = True

    def __init__(self, *args, **kwargs):
        super(XmlField, self).__init__(*args, **kwargs)
    
    def __serialize__(self, obj, xml):                
        """Parse the value of this field as xml and add nodes to the serializer tree"""
        value = self.value_to_string(obj).encode('utf-8')
        parser = sax.make_parser(["easymode.utils.xmlutils"])
        parser.setContentHandler(xml)
        parser.parse(StringIO(value))



"""
    easy_reset_language <target locale> <app>
    
    will clear all fields for the locale in question so the values will be read from
    the locale again.
    
    example:
    
    ./manage.py easy_reset_language en myapp.mymodel
    
    This will clear myapp.mymodel in the en locale so all values
    will be fetched by gettext instead of being overridden.
"""
from django.conf import settings
from django.core.management.base import BaseCommand
from django.db.models import get_models, get_app, get_model
from django.utils import translation
from django.utils.encoding import force_unicode

from easymode import i18n
from easymode.utils.languagecode import get_real_fieldname


class Command(BaseCommand):
    help = __doc__
    
    def handle(self, locale, app=None, **options):
        """ command execution """
        translation.activate(settings.LANGUAGE_CODE)
        if app:
            unpack = app.split('.')

            if len(unpack) == 2:
                models = [get_model(unpack[0], unpack[1])]
            elif len(unpack) == 1:
                models = get_models(get_app(unpack[0]))
        else:
            models = get_models()
            
        for model in models:
            if hasattr(model, 'localized_fields'):
                i18n.unregister(model)
                model_full_name = '%s.%s' % (model._meta.app_label, model._meta.module_name)
                update_instances = set()
                messages = []
                
                for instance in model.objects.all():
                    for field in model.localized_fields:
                        local_field_name = get_real_fieldname(field, locale)
                        
                        if  hasattr(instance, local_field_name):
                                local_field_value = getattr(instance, local_field_name)
                                if local_field_value not in (None, u''):
                                    setattr(instance, local_field_name, None)
                                
                                    update_instances.add(instance)
                                    messages.append(u"%s %s %s : %s will become reset to None" % (model_full_name, instance, local_field_name, force_unicode(local_field_value)))

                if len(update_instances):
                    if self.ask_for_confirmation(messages, u'%s.%s' % (model._meta.app_label, model._meta.module_name)):
                        for update_instance in update_instances:
                            print u"resetting %s" % update_instance
                            update_instance.save()
                
                
    def ask_for_confirmation(self, messages, model_full_name):
        print u'\nSummary for "%s":' % model_full_name
        for message in messages:
            print '   %s' % message
        while True:
            prompt = u'\nAre you sure that you want to reset %s: (y/n) [n]: ' % model_full_name
            answer = raw_input(prompt).strip()
            if answer == '':
                return False
            elif answer not in ('y', 'n', 'yes', 'no'):
                print u'Please answer yes or no'
            elif answer == 'y' or answer == 'yes':
                return True
            else:
                return False
            



"""
Value objects used by easymode's i18n.meta package.
"""
class GettextVO:
    """
    A value object that contains information about the origin
    of the value in question.
    
    .. attribute:: standin_value_is_from_database

        is True if the value came from the database

    .. attribute:: msgid
    
        The message id that originated the lookup

    .. attribute:: msg
        
        The message that is the result of the lookup
    
    .. attribute:: fallback
    
        the fallback that was found during the lookup
    
    .. attribute:: stored_value
        
        the value as it is stored in the database
    
    """
    
    def __init__(self):
        self.standin_value_is_from_database = False
        self.msgid = None
        self.msg = None
        self.fallback = None
        self.stored_value = None



from django.contrib.admin.widgets import RelatedFieldWidgetWrapper
from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape

__all__ = ('WidgetWrapper',)

def find_extra_attrs(value):
    "finds extra attributes in *value* if they are there"
    extra = None
    value_is_from_database = True

    if hasattr(value, 'standin_value_is_from_database'):
        value_is_from_database = value.standin_value_is_from_database

        if value.stored_value != value.msg:
            extra = value.msg
        elif value.stored_value != value.fallback:
            extra = value.fallback
    
    return extra, value_is_from_database

class WidgetWrapper(RelatedFieldWidgetWrapper):
    """
    This class is a wrapper to a given widget to add a marker next to
    the field, so you can see that the field is translated.
    """
    def __init__(self, widget, **kwargs):            
        if type(widget) is WidgetWrapper:
            self.widget = widget.widget
        else:
            self.widget = widget
        
        self.is_hidden = self.widget.is_hidden
        self.needs_multipart_form = self.widget.needs_multipart_form
        self.attrs = self.widget.attrs
        self.choices = getattr(self.widget, 'choices', None)

    def render(self, name, value, *args, **kwargs):
        (extra, value_is_from_database) = find_extra_attrs(value)
        widget_html = self.widget.render(name, value, *args, **kwargs)
        
        if extra and value_is_from_database:
            return mark_safe(u'<div class="localized catalog-has-different-data">%s <small><a class="extra-catalog-data" title="%s">\u2234\u207A</a></small></div>' % (widget_html, conditional_escape(extra)))
        elif not value_is_from_database:
            return mark_safe(u'<div class="localized">%s <small>\u2234\u00B0</small></div>' % widget_html)
        
        return mark_safe(u'<div class="localized">%s <small>\u2234</small></div>' % widget_html)



from django.contrib import admin
from django.forms.formsets import BaseFormSet
from django.test import TestCase

from easymode.tests.admin import TestModelAdmin, SelfAwareTestModelAdmin
from easymode.tests.models import TestModel
from easymode.tests.testcases import initdb
from easymode.tree.admin.widgets import foreignkey


__all__ = ('TestOldRelatedAdmin',)

@initdb
class TestOldRelatedAdmin(TestCase):
    
    fixtures = ['auth-user']
    
    def setUp(self):
        t = TestModel(charfield='Hoi Ik ben de root node')
        t.save()
        f = t.submodels.create(subcharfield="Hoi Ik ben de first level sub node", subintegerfield=10)
        s = t.submodels.create(subcharfield="Hoi ik ben de tweede first level sub node", subintegerfield=100)
        w = t.secondsubmodels.create(ultrafield="Sed tempor. Ut felis. Maecenas erat.")
        f.subsubmodels.create(subsubcharfield="Hoi ik ben de thord level sub node")
        f.subsubmodels.create(subsubcharfield="Hoi ik ben de third level sub node")
        s.subsubmodels.create(subsubcharfield="Hoi ik ben de third level sub node")
        s.subsubmodels.create(subsubcharfield="Hoi ik ben de third level sub node")        
        t.save()
    
    
    def test_related_admin(self):
        rel = TestModelAdmin(TestModel, admin.site)
        formset_array = rel.extra_forms(1)
        assert(isinstance(formset_array[0], BaseFormSet))
    
    def test_there_are_two_forms(self):
        """because there are two models marked as chlidren in the modelamin"""
        rel = TestModelAdmin(TestModel, admin.site)
        formset_array = rel.extra_forms(1)
        assert(len(formset_array) == 2)
        
    def test_self_aware_related_admin(self):
        rel = SelfAwareTestModelAdmin(TestModel, admin.site)
        formset_array = rel.extra_forms(1)
        assert(isinstance(formset_array[0], BaseFormSet))
    
    def test_admin_with_extra_fields_are_displayed(self):
        response = self.client.login(username='admin', password='admin')
        response = self.client.get('/admin/tests/testmodel/1/')
        self.assertTemplateUsed(response, 'tree/admin/widgets/foreignkeylink.html')
        self.assertContains(response, 'Sed tempor. Ut felis. Maecenas erat.')
    
    def test_there_are_more_than_one_forms(self):
        rel = SelfAwareTestModelAdmin(TestModel, admin.site)
        formset_array = rel.extra_forms(1)
        assert(isinstance(formset_array[0], BaseFormSet))
        assert(isinstance(formset_array[1], BaseFormSet))
        
    def test_admin_template_for_change_is_different(self):        
        response = self.client.login(username='admin', password='admin')
        response = self.client.get('/admin/tests/testmodel/1/')
        self.assertTemplateUsed(response,'tree/admin/change_form_with_related_links.html')
        



from django.template.defaultfilters import stringfilter
from django import template

from easymode.utils import languagecode

register = template.Library()

@register.filter
@stringfilter
def strip_locale(url):
    """Strip the language code from the beginning of a string"""
    return languagecode.strip_language_code(url)

@register.filter
def fix_locale_from_request(url, current_language):
    """
    Fixes the language code as follows:
    
    If there is only one language used in the site, it strips the language code.
    If there are more languages used, it will make sure that the url has the current
    language as a prefix.
    """
    return languagecode.fix_language_code(url, current_language)

@register.filter
@stringfilter
def fix_shorthand(language_code):
    """
    Use this if you want to use settings.USE_SHORT_LANGUAGE_CODES.
    fix_shorthand will always return the correct locale to use in an url,
    depending on your settings of USE_SHORT_LANGUAGE_CODES.
    """
    return languagecode.get_shorthand_from_language_code(language_code)





"""
Functionality to create trees of django models.

If a django model tree can be categorised as a DAG (Directed Acylclic Graph)
This module contains the machinery to add recursive serialisation to xml
and recursive admin support to the models in such a tree.
"""

__all__ = ('xml', 'admin', 'decorators', 'introspection', 'query', 'serializers')

def xml(obj):
    """
    Used to convert either 
    
    - a django model decorated with :func:`easymode.tree.xml.decorators.toxml`
    - a queryset obtained by querying a model decorated with :func:`easymode.tree.xml.decorators.toxml`
    - an :class:`easymode.tree.xml.query.XmlQuerySetChain`
    
    to xml
    """
    return (obj.__xml__())


import re
import time

from django.conf import settings
from django.contrib.sessions.middleware import SessionMiddleware
from django.middleware.locale import LocaleMiddleware
from django.utils import translation
from django.utils.http import cookie_date

from easymode.utils.languagecode import get_short_language_codes,\
    get_language_code_from_shorthand,\
    get_shorthand_from_language_code as language_as_slug


USE_SHORT_LANGUAGE_CODES = getattr(settings, 'USE_SHORT_LANGUAGE_CODES', False)

################################################################################
# Compiled regular expressions
################################################################################

MATCH_LANGUAGE_CODE = re.compile(
    r"^/(%s)/.*" % "|".join(map(lambda l: l[0], settings.LANGUAGES)))
MATCH_SHORT_LANGUAGE_CODE = re.compile(
    r"^/(%s)/.*" % "|".join(get_short_language_codes()))

if USE_SHORT_LANGUAGE_CODES:
    HREF_REGEX = re.compile(
        ur'<a([^>]+)href="/(?!(%s|%s|%s))([^"]*)"([^>]*)>' % (
            "|".join(map(lambda l: l + "/" , get_short_language_codes())), 
            settings.MEDIA_URL[1:], 
            settings.ADMIN_MEDIA_PREFIX[1:]
        )
    )
    FORM_REGEX = re.compile(
        ur'<form([^>]+)action="/(?!(%s|%s|%s))([^"]*)"([^>]*)>' % (
            "|".join(map(lambda l: l + "/" , get_short_language_codes())),
             settings.MEDIA_URL[1:],
             settings.ADMIN_MEDIA_PREFIX[1:]
        )
    )
else:
    HREF_REGEX = re.compile(
        ur'<a([^>]+)href="/(?!(%s|%s|%s))([^"]*)"([^>]*)>' % (
            "|".join(map(lambda l: l[0] + "/" , settings.LANGUAGES)), 
            settings.MEDIA_URL[1:], 
            settings.ADMIN_MEDIA_PREFIX[1:]
        )
    )
    FORM_REGEX = re.compile(
        ur'<form([^>]+)action="/(?!(%s|%s|%s))([^"]*)"([^>]*)>' % (
            "|".join(map(lambda l: l[0] + "/" , settings.LANGUAGES)),
             settings.MEDIA_URL[1:],
             settings.ADMIN_MEDIA_PREFIX[1:]
        )
    )

################################################################################
# helper functions
################################################################################

def has_lang_prefix(path):
    if USE_SHORT_LANGUAGE_CODES:
        check = MATCH_SHORT_LANGUAGE_CODE.match(path)
        
        if check is not None:
            return get_language_code_from_shorthand(check.group(1))
    else:
        check = MATCH_LANGUAGE_CODE.match(path)
        if check is not None:
            return check.group(1)
        
        return False

################################################################################
# middlewares
################################################################################

class NoVaryOnCookieSessionMiddleWare(SessionMiddleware):
    """
    Works like django's :class:`django.middleware.SessionMiddleWare`, but it
    does not add cookie to the vary header. You need this if you use google
    analytics and want to use django site caching.
    """

    def process_response(self, request, response):
        """
        If ``request.session was modified``, or if the configuration is to save
        the session every time, save the changes and set a session cookie.
        """
        try:
            modified = request.session.modified
        except AttributeError:
            pass
        else:
            if modified or settings.SESSION_SAVE_EVERY_REQUEST:
                if request.session.get_expire_at_browser_close():
                    max_age = None
                    expires = None
                else:
                    max_age = request.session.get_expiry_age()
                    expires_time = time.time() + max_age
                    expires = cookie_date(expires_time)
                # Save the session data and refresh the client cookie.
                request.session.save()
                response.set_cookie(settings.SESSION_COOKIE_NAME,
                        request.session.session_key, max_age=max_age,
                        expires=expires, domain=settings.SESSION_COOKIE_DOMAIN,
                        path=settings.SESSION_COOKIE_PATH,
                        secure=settings.SESSION_COOKIE_SECURE or None)
        return response

class LocaleFromUrlMiddleWare(LocaleMiddleware):
    """
    Like :class:`django.middleware.LocaleMiddleware` this middleware activates
    the current language. It does not try to guess the language from the request
    headers, it looks for it in the url, or it defaults to
    ``settings.LANGUAGE_CODE``.
    
    Also we don't use the accept language to determine the language of the page
    anymore, so the Accept-Language is nolonger considered for the vary headers.
    """
    def process_request(self, request):
        language = has_lang_prefix(request.path_info)
        
        if not language:
            language = settings.LANGUAGE_CODE
        
        if hasattr(request, "session"):
            request.session["django_language"] = language
        else:
            request.set_cookie("django_language", language)
        
        translation.activate(language)
        request.LANGUAGE_CODE = translation.get_language()
    
    def process_response(self, request, response):
        if 'Content-Language' not in response:
            response['Content-Language'] = translation.get_language()
        translation.deactivate()
        return response
    

class LocaliseUrlsMiddleware(object):
    """
    This middleware replaces all anchor tags with localised versions, ugly
    but it works.
    
    Don't put any vary header for the Accept-Language because the language does
    not depend on the vary header, the language is in the url.
    """
    
    def process_response(self, request, response):
        translation.deactivate()
        path = unicode(request.path)

        if not path.startswith(settings.MEDIA_URL) and \
            not path.startswith(settings.ADMIN_MEDIA_PREFIX) and \
            response.status_code == 200 and \
            response._headers['content-type'][1].split(';')[0] == "text/html":
                
            response.content = HREF_REGEX.sub(
                ur'<a\1href="/%s/\3"\4>' % language_as_slug(request.LANGUAGE_CODE), 
                response.content.decode('utf-8'))
            response.content = FORM_REGEX.sub(
                ur'<form\1action="/%s/\3"\4>' % language_as_slug(request.LANGUAGE_CODE), 
                response.content.decode('utf-8'))
                
        if (response.status_code == 301 or response.status_code == 302 ):
            location = response._headers['location']
            prefix = has_lang_prefix(location[1])
            if not prefix and location[1].startswith("/") and \
               not location[1].startswith(settings.MEDIA_URL) and \
               not location[1].startswith(settings.ADMIN_MEDIA_PREFIX):
                response._headers['location'] = (
                    location[0], "/%s%s" % (
                        language_as_slug(request.LANGUAGE_CODE), location[1]
                    )
                )
        return response

################################################################################
# Shorthand versions of locale middlewares.
# These versions only add the language part of the langugae code and omit the 
# country
################################################################################

class ShortLocaleFromUrlMiddleWare(LocaleFromUrlMiddleWare):
    """
    deprecated, :class:`LocaleFromUrlMiddleWare` does exactly the same, if
    you've set ``USE_SHORT_LANGUAGE_CODES`` to ``True``.
    """
    def process_request(self, request):
        super(ShortLocaleFromUrlMiddleWare, self).process_request(request)

class ShortLocaleLocaliseUrlsMiddleware(LocaliseUrlsMiddleware):
    """
    deprecated, :class:`LocaliseUrlsMiddleware` does exactly the same, if
    you've set ``USE_SHORT_LANGUAGE_CODES`` to ``True``.
    """
    def process_response(self, request, response):
        return super(ShortLocaleLocaliseUrlsMiddleware, self).process_response(request, response)



"""
Contains classes and functions for dealing with html entities.

You need this as soon as you are doing anything with rich
text fields and want to use :func:`easymode.tree.xml.decorators.toxml`.
"""
import re
import StringIO
import htmlentitydefs

from django.utils.xmlutils import SimplerXMLGenerator

from xml.sax.expatreader import ExpatParser
from xml.sax import SAXParseException
from xml.sax.handler import EntityResolver, ContentHandler

entities = re.compile(r'&([^;]+);')

__all__ = ('unescape_all', 'XmlScanner', 'XmlPrinter', 'create_parser')

def _unicode_for_entity_with_name(name):
    # if the entity is unknown, insert question mark
    codepoint = htmlentitydefs.name2codepoint.get(name, 63)
    return unichr(codepoint)
   
def unescape_all(string):
    """Resolve all html entities to their corresponding unicode character"""
    def escape_single(matchobj):
        return _unicode_for_entity_with_name(matchobj.group(1))
    return entities.sub(escape_single, string)

def is_valid(xml_string):
    """validates a unicode string containing xml"""
    xml_file = StringIO.StringIO(xml_string.encode('utf-8'))
    
    parser = XmlScanner()
    parser.setContentHandler(ContentHandler())
    try:
        parser.parse(xml_file)
    except SAXParseException:
        return False
    
    return True
    
class XmlScanner(ExpatParser):
    """
    This class is an xml parser that will not throw errors
    when unknown entities are found.
    
    It can be used as follows::
    
        parser = sax.make_parser(["easymode.utils.xmlutils"])
    
    All entities that can not be resolved will be skipped.
    This will trigger skippedEntity on the contentHandler.
    
    A good contenthandler for this scanner is
    :class:`~easymode.utils.xmlutils.XmlPrinter` because it
    will turn these entities into their unicode characters.
    """
    def __init__(self, *args, **kwargs):
        """docstring for __init__"""
        ExpatParser.__init__(self, *args, **kwargs)
        
        class _NiftyEntityResolver(EntityResolver):
            """
            _NiftyEntityResolver makes the entity errors disappear.
            
            It works by the html spec which says that external entities that
            can not resolved, do not have to be reported as an error by a non
            validating parser.
            """
            def resolveEntity(self, publicId, systemId):
                "This will be called with None, None if it is an unknown entity"
                if systemId or publicId:
                     return EntityResolver.resolveEntity(self,publicId,systemId)
                
                # return an empty stringio in which the parser can never find
                # the entities. This will make it skip the entity, which triggers
                # skippedEntity in the contenthandler.
                return StringIO.StringIO(" ") # if you delete that single space you will be sorry.
        
        self.setEntityResolver(_NiftyEntityResolver())
        
    def reset(self):
        ExpatParser.reset(self)
        # This line makes the parser think unknown
        # entities are external.
        self._parser.UseForeignDTD(True)
    
   
    
class XmlPrinter(SimplerXMLGenerator):
    """
    XmlPrinter can be used as a contenthandler for the 
    :class:`~easymode.utils.xmlutils.XmlScanner`.
    
    It will convert all skippedEntities to their unicode characters
    and copy these to the output stream.
    
    You can use it as a simple xml generator, to create xml::
    
        stream = StringIO.StringIO()
        xml = XmlPrinter(stream, settings.DEFAULT_CHARSET)

        def _render_page(page):
            xml.startElement('title', {})
            xml.characters(page.title)
            xml.endElement('title')
            xml.startElement('template', {})
            xml.characters(page.template)
            xml.endElement('template')
        
        for child in Page.children.all():
            xml.startElement('page', {})
            _render_page(child)
            xml.endElement('page')
    """
    def skippedEntity(self, name):
        self._out.write(_unicode_for_entity_with_name(name).encode('utf-8'))

def create_parser(*args, **kwargs):
    """
    Because this function is defined, you can create an 
    :class:`~easymode.utils.xmlutils.XmlScanner` like this::
        
        from xml import sax
        
        parser = sax.make_parser(["easymode.utils.xmlutils"])
    """
    return XmlScanner(*args, **kwargs)



from django.contrib import admin
from django.contrib.admin.sites import AlreadyRegistered
from django.db.models.base import ModelBase

from easymode.i18n.admin.decorators import L10n

__all__ = ('register_all',)

def register_all(module, exclude=[], AdminClass=admin.ModelAdmin):
    """
    Register all models in *module* with the admin
    
    :param module: The module that contains the models
    :param exclude: A list of models classes that should not be registered
    """
    member_names = dir(module)

    for member_name in member_names:
        obj = getattr(module, member_name, None)
        if isinstance(obj, ModelBase) \
            and obj not in exclude \
            and not obj._meta.abstract:
            try:
                if hasattr(obj, 'localized_fields'):
                    admin.site.register(obj, L10n(obj, AdminClass))
                else:
                    admin.site.register(obj, AdminClass)
            except AlreadyRegistered:
                pass


# Create your views here.
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext

from easymode.tree import xml as tree
from easymode.tree.xml.query import XmlQuerySetChain
from easymode.xslt.response import render_to_response as render_xslt_to_response
from foobar import models as foobar_models

def index(request):
    """shows an informative page"""
    return render_to_response('index.html', {}, context_instance=RequestContext(request))
    
def raw(request):
    """shows untransformed hierarchical xml output"""
    foos = foobar_models.Foo.objects.all()
    return HttpResponse(tree.xml(foos), mimetype='text/xml')

def chain(request):
    """shows how the XmlQuerySetChain can be used instead of @toxml decorator"""
    bars = foobar_models.Bar.objects.all()
    bazs = foobar_models.Baz.objects.all()
    qsc = XmlQuerySetChain(bars, bazs)
    return HttpResponse(tree.xml(qsc), mimetype='text/xml')
    
def xslt(request):
    """Shows xml output transformed with standard xslt"""
    foos = foobar_models.Foo.objects.all()
    return render_xslt_to_response('xslt/model-to-xml.xsl', foos, mimetype='text/xml')

def frontend(request, format='html'):
    if format == 'xml':
        return xslt(request)
    
    foos = foobar_models.Foo.objects.all()
    return render_xslt_to_response('frontend.xsl', foos)


"""
Generic django admin extensions.
"""

from django.contrib import admin

__all__ = ('DragAndDropAdmin', 'abstract', 'fields', 'forms', 'models', 'utils')

class DragAndDropAdmin(admin.ModelAdmin):
    """
    Admin class that allows dragging and dropping of items
    in the admin listing.
    
    Go to http://jqueryui.com/download, deselect all and then select sortable.
    Download the js and include it in the Media of your derived class::
        
        class Media:
            js = (
                'js/jquery-ui-1.8.14.custom.js',
            )
    
    Also make sure jquery itself is loaded.
    """
    ordering = ('order',)
    list_display = ('id', 'order')
    list_editable = ('order',)
    
    class Media:
        js = (
            'easymode/js/menu-sort.js',            
        )
        
        



#!/usr/bin/env python
import os
import re
from distutils.core import setup
from distutils.command.install import INSTALL_SCHEMES

description="""With easymode you can create backends for dynamic flash/flex
websites.Easymode makes internationalization simple and outputs xml by
default. To tailor the xml to your application, you can transform
it using xslt templates, which easymode integrates.

For more info, look at http://packages.python.org/django-easymode/details.html

documentation at http://packages.python.org/django-easymode/

release notes at http://packages.python.org/django-easymode/changes.html"""

version = '1.4b2'
packages = []
data_files = []

for scheme in INSTALL_SCHEMES.values():
    scheme["data"] = scheme["purelib"]

def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

PYC = re.compile(r'.*\.py[co]$')

for dirpath, dirnames, filenames in os.walk('easymode'):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)))
    elif filenames:
        data_files.append([dirpath, 
            [os.path.join(dirpath, f) for f in filenames if not PYC.match(f)]
        ])

setup(name='django-easymode',
    version=version,
    description='Quickly build backends for flash/flex websites with django',
    author='L. van de Kerkhof',
    author_email='easymode@librelist.com',
    maintainer='L. van de Kerkhof',
    maintainer_email='easymode@librelist.com',
    keywords='adobe flex flash xml xslt i18n internationalization translate django',
    long_description=description,
    install_requires=[
        'django>=1.4',
        'polib',
    ],
    url='http://github.com/specialunderwear/django-easymode',
    packages=packages,
    data_files=data_files,
    platforms = "any",
    license='GPL',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
        'Topic :: Software Development :: Localization',
    ],
)



"""
Some extensions to polib (http://bitbucket.org/izi/polib)
"""
import types

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


if 'rosetta' in settings.INSTALLED_APPS:
    from rosetta import polib
else:
    try:
        import polib
    except ImportError:
        raise ImproperlyConfigured(
            """
            Please install either django-rosetta: 
            http://code.google.com/p/django-rosetta/
            or polib:
            http://bitbucket.org/izi/polib/src/
            otherwise easymode.utils.polibext won't work"""
        )
        

def po_to_unicode(po_obj):
    """
    Turns a polib :class:`polib.PoFile` or a :class:`polib.PoEntry` 
    into a :class:`unicode` string.
    
    :param po_obj: Either a :class:`polib.PoFile` or :class:`polib.PoEntry`.
    :rtype: :class:`unicode` string.
    """
    po_text = po_obj.__str__()
    if type(po_text) != types.UnicodeType:
        po_text = po_text.decode('utf-8')
    
    return po_text
    
class PoStream(polib._POFileParser):
    """
    Create a POFile object from a :class:`~StringIO.StringIO`
    instead of a file name or handle.::
    
        import codecs
        from StringIO import StringIO
        
        po_string = codecs.open('sompofile.po', 'r', 'utf-8').read()
        po_file_from_string = PoStream(StringIO(po_string)).parse()
        
        unicode(po_file_from_string)
    
    """
    
    def __init__(self, stream, fpath=None, encoding='utf-8', wrapwidth=78):
        """a pofileparser that can read from a stream"""
        self.fhandle = stream
        self.instance = polib.POFile(fpath=fpath, encoding=encoding)
        self.instance.wrapwidth = wrapwidth
        self.transitions = {}
        self.current_entry = polib.POEntry()
        self.current_state = 'ST'
        self.current_token = None
        self.msgstr_index = 0
        self.entry_obsolete = 0
        all_ = ['ST', 'HE', 'GC', 'OC', 'FL', 'TC', 'MS', 'MP', 'MX', 'MI']

        self.add('TC', ['ST', 'HE'],                                     'HE')
        self.add('TC', ['GC', 'OC', 'FL', 'TC', 'MS', 'MP', 'MX', 'MI'], 'TC')
        self.add('GC', all_,                                             'GC')
        self.add('OC', all_,                                             'OC')
        self.add('FL', all_,                                             'FL')
        self.add('MI', ['ST', 'HE', 'GC', 'OC', 'FL', 'TC', 'MS', 'MX'], 'MI')
        self.add('MP', ['TC', 'GC', 'MI'],                               'MP')
        self.add('MS', ['MI', 'MP', 'TC'],                               'MS')
        self.add('MX', ['MI', 'MX', 'MP', 'TC'],                         'MX')
        self.add('MC', ['MI', 'MP', 'MS', 'MX'],                         'MC')



from django.core.urlresolvers import reverse
from django.test import TestCase

from easymode.tests.testcases import initdb


__all__ = ('TestSuperRecusionLoop',)

@initdb
class TestSuperRecusionLoop(TestCase):
    
    fixtures = ['managererrormodel', 'auth-user', 'auth-group',]
    
    def test_recursion_error_by_super(self):
        "Calling super of an admin method should not cause infinite recursion"
        self.assertTrue( self.client.login(username='admin', password='admin') )
        
        admin_url = reverse('admin:tests_managererrormodel_delete', args=(1,))
        try:
            response = self.client.post(admin_url)
            self.failUnlessEqual(response.status_code, 200)
        except RuntimeError as e:
            self.fail(e)
    


"""
Fields to be used in forms.
"""
import re

from django.forms import ValidationError
from django.forms import fields
from django.utils.encoding import smart_unicode
from django.utils.html import strip_tags

class PoSafeTextField(fields.CharField):
    """
    Having carriage return in po files is an error.
    This field's clean function will strip these ugly
    characters from the input.
    """

    def clean(self, value):
        if value is not None:
            safe_value = value.replace('\r','')
        else:
            safe_value = value
            
        return super(PoSafeTextField, self).clean(safe_value)
    
class HtmlEntityField(fields.CharField):
    """
    A form field that can validate the length of a rich text field.
    
    It ignores html markup and it counts html entities as only a single
    character.
    """
    def clean(self, value):
        if value is not None:
            value = value.replace('\r','')
            tagless_value = strip_tags(value)
            entityless_value = re.sub(r'&[^;]+;', 'X', tagless_value)
        else:
            entityless_value = value
            value = u''
        
        # validate using super class
        super(HtmlEntityField, self).clean(entityless_value)
        # return unmodified value, but with normalized line endings.
        return smart_unicode(value)

class FlashUrlField(fields.URLField):
    """
    A field that validates input to be either 
    
    1. an absolute url, eg. *http://example.com/*
    2. a relative url, eg. */main/chapter* 
    3. a flash url with a hash eg. *#/main/chapter*
    """

    def clean(self, value):
        try:
            cleaned_value = super(FlashUrlField, self).clean(value)
        except ValidationError:
            if re.match(r'^#/', value):
                return value
            elif re.match(r'^/\S+$', value):
                return value
            else:
                raise ValidationError(self.error_messages['invalid'])

        return cleaned_value



import os.path
import shutil
from functools import wraps

from django.conf import settings

from easymode.tests.testutils import TestSettingsManager


def initdb(cls):
    """
    This is a class decorator.

    Adds a setUp and tearDown method to a TestCase class.
    The setup will add tests to the installed apps, causing
    the models in the tests folder to be loaded.

    Then at the end they are removed again.
    """

    orig_pre_setup = getattr(cls, '_pre_setup')
    orig_post_teardown = getattr(cls, '_post_teardown')

    @wraps(orig_pre_setup)
    def _pre_setup(self):
        self.settingsManager = TestSettingsManager()
        self.settingsManager.set(INSTALLED_APPS=settings.INSTALLED_APPS + (
            'easymode',
            'easymode.tests',
            ),
        )

        for skipped_test in getattr(settings, 'SKIPPED_TESTS', []):
            if hasattr(cls, skipped_test):
                setattr(cls, skipped_test, lambda x: True)

        if orig_pre_setup:
            orig_pre_setup(self)

    @wraps(orig_post_teardown)
    def _post_teardown(self):
        if orig_post_teardown:
            orig_post_teardown(self)

        # don't try to remove the locale dir in the project dir because it has
        # all translations
        # test cases should have a separate locale dir, preferably in /tmp
        if not (os.path.normpath(settings.LOCALE_DIR) == \
            os.path.normpath(settings.PROJECT_DIR)):
            locale_dir = os.path.join(settings.LOCALE_DIR, 'locale')
            if os.path.exists(locale_dir):
                shutil.rmtree(locale_dir)
        self.settingsManager.revert()

    cls._pre_setup = _pre_setup
    cls._post_teardown = _post_teardown
    return cls



import errno
import os
from hashlib import sha1
from os.path import join

from django.conf import settings
from django.test import TestCase

from easymode.utils import mutex, SemaphoreException, recursion_depth


__all__ = ('TestUtils',)

class TestUtils(TestCase):
    
    def test_semaphore(self):
        """the semaphore should bind a file to the context and hold on to it untill after the context is closed"""
        lockfile = join('/tmp', sha1(settings.SECRET_KEY).hexdigest() +'.semaphore')
        for a in range(1, 4):
            with mutex():
                assert(os.path.isfile(lockfile))
        
            assert(not os.path.exists(lockfile))

    def test_semaphore_exception(self):
        """The semaphore should throw an exception when the wait time is over"""
        lockfile = join('/tmp', sha1(settings.SECRET_KEY).hexdigest() +'.semaphore')
        try:
            with mutex(max_wait=0, lockfile=lockfile):
                with mutex(max_wait=0, lockfile=lockfile):
                    pass
        except SemaphoreException:
            pass
        else:
            self.fail('Should have triggered SemaphoreException')
        self.assertFalse(os.path.exists(lockfile))
        
    def test_fdopen_works_correctly(self):
        """fdopen should allow proper locking"""
        lockfile = join('/tmp', sha1(settings.SECRET_KEY).hexdigest() +'.semaphore')
        
        try:
            os.open(lockfile, os.O_EXCL | os.O_RDWR | os.O_CREAT)
            os.open(lockfile, os.O_EXCL | os.O_RDWR | os.O_CREAT)
        except OSError as e:
            self.assertEqual(e.errno, errno.EEXIST)
        else:
            self.fail("If the file exists an OSError should be thrown")
        finally:
            os.remove(lockfile)
    
    def test_recursion_depth(self):
        """recursion depth should work properly"""
        def recurse(arr):
            with recursion_depth('test_recursion_depth') as recursion_level:
                if recursion_level > 10:
                    raise Exception('error')
                if arr and arr.pop():
                    recurse(arr)

        recurse(range(0,10))
        self.assertRaises(Exception, recurse, range(0,11))
        recurse(range(0,10))
        self.assertRaises(Exception, recurse, range(0,11))
        



"""
utility functions used by easymode's i18n.meta package to get localized attributes
from a class.
"""
from django.conf import settings
from django.utils import translation

from easymode.utils import first_match
from easymode.utils.languagecode import get_real_fieldname
from easymode.utils import first_match


def valid_for_gettext(value):
    """Gettext acts weird when empty string is passes, and passing none would be even weirder"""
    return value not in (None, "")
    
def get_fallback_languages():
    """Retrieve the fallback languages from the settings.py"""
    lang = translation.get_language()
    fallback_list = settings.FALLBACK_LANGUAGES.get(lang, None)
    if fallback_list:
        return fallback_list

    return settings.FALLBACK_LANGUAGES.get(lang[:2], [])


def get_localized_property(context, field=None, language=None):
    '''
    When accessing to the name of the field itself, the value
    in the current language will be returned. Unless it's set,
    the value in the default language will be returned.
    '''
    if language:
        return getattr(context, get_real_fieldname(field, language))
    
    if hasattr(settings, 'FALLBACK_LANGUAGES'):
        attrs = [translation.get_language()]
        attrs += get_fallback_languages()
    else:
        attrs = [
            translation.get_language(),
            translation.get_language()[:2],
            settings.LANGUAGE_CODE, 
        ]
    
    def predicate(x):
        value = getattr(context, get_real_fieldname(field, x), None)
        return value if valid_for_gettext(value) else None

    return first_match(predicate, attrs)


def get_localized_field_name(context, field):
    """Get the name of the localized field"""
    attrs = [
             translation.get_language(), 
             translation.get_language()[:2], 
             settings.LANGUAGE_CODE
            ]
            
    def predicate(x):
        field_name = get_real_fieldname(field, x)
        if hasattr(context, field_name):
            return field_name
        return None

    return first_match(predicate, attrs)

def get_field_from_model_by_name(model_class, field_name):
    """
    Get a field by name from a model class without messing with the app cache.
    """
    return first_match(lambda x: x if x.name == field_name else None, model_class._meta.fields)
